from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _
from .global_vars import config


class ConfVarConfig(AppConfig):
    name = 'django_admin_conf_vars'
    verbose_name = _(u'Global configuration variables')

    def ready(self):
        config.load_attributes()



from django.contrib import admin
from .models import ConfVar


class ConfVarAdmin(admin.ModelAdmin):
    list_display = ('name', 'value',)
    list_editable = ('value',)
    fields = ('name', 'value',)


    def get_readonly_fields(self, request, obj=None):
        if obj:
            return ('name',) +  self.readonly_fields
        return self.readonly_fields

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

admin.site.register(ConfVar, ConfVarAdmin)



from django.conf import settings
from .models import ConfVar
import os
import importlib
DEFAULT = 'global_vars'


class SingletonVar(object):

    ATTRIBUTES = {}

    def __new__(cls, *a, **k):
        if not hasattr(cls, '_inst'):
            cls._inst = super(SingletonVar, cls).__new__(cls, *a, **k)
        return cls._inst

    def __getattr__(self, key):
        if not bool(len(self.ATTRIBUTES)):
            self.load_attributes()
        return self.ATTRIBUTES.get(key, None)

    def getAll(self):
        if not bool(len(self.ATTRIBUTES)):
            self.load_attributes()
        return eval(str(self.ATTRIBUTES))

    def set(self, name, default=0):
        var, created = ConfVar.objects.get_or_create(name=name)
        if created:
            var.value = str(default)
            var.save(reload=False)
        self.ATTRIBUTES[var.name] = var.value

    def reload(self, name, value):
        self.ATTRIBUTES[name] = value

    def load_attributes(self):
        PROJECT_ROOT = os.environ['DJANGO_SETTINGS_MODULE'].split('.')[0]
        try:
            vars_path = settings.GLOBAL_VARS_PATH
        except Exception:
            vars_path = DEFAULT

        __import__(PROJECT_ROOT, fromlist=[vars_path])


config = SingletonVar()



from django.db import models
from django.utils.translation import ugettext_lazy as _


class ConfVar(models.Model):
    name = models.CharField(max_length=100)
    value = models.CharField(max_length=255)
    
    def save(self, reload=True, *args, **kwargs):
        res = super(ConfVar, self).save(*args, **kwargs)
        if reload:
            from .global_vars import config
            config.reload(self.name, self.value)
        return res 

    def get_absolute_url(self):
        return self.name

    def __unicode__(self):
        return u"%s: %s" %(self.name, self.value)

    class Meta:
        verbose_name = _(u'Variable')
        verbose_name_plural = _(u'Variables list')


import os
from setuptools import setup, find_packages

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-admin-conf-vars',
    version='0.1',
    include_package_data=True,
    license='BSD License',
    description='A simple Django app to have configuration variables for your project with the Django admin.',
    long_description=README,
    url='https://github.com/maoaiz/django-admin-conf-vars',
    packages=find_packages(),
    install_requires=['django>=1.7', ],
    author='Mauricio Aizaga',
    author_email='mauricioaizaga@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)